# -*- coding: utf-8 -*-


# import utils_fit
import fit_T_dep
import matplotlib.pyplot as plt
import numpy as np
import scipy.constants

KB = scipy.constants.Boltzmann
hbar = scipy.constants.hbar

# --- plotting parameters --- #

import scienceplots

plt.close("all")
plt.style.use(["science"])

# --- setup figure --- #

# sample is mpaf13_9_19

mm = 1 / 25.4
fig_width = 80
ratio = (1 + np.sqrt(5)) / 2
fig_height = fig_width / ratio

plt.close("all")
fig, ax = plt.subplots(
    2,
    1,
    figsize=(fig_width * mm, 2 * fig_height * mm),
    constrained_layout=True,
    sharex=True,
)
ax1, ax2 = ax

verbose = False

# --- fitting methods --- #


def find_nearest(array, value):
    array = np.asarray(array)
    idx = np.argmin(np.abs(array - value))
    return idx


from lmfit import Minimizer, Parameters

T_arr = np.array(
    [1.9, 2.1, 2.3, 2.5, 2.75, 3.0, 3.25, 3.5, 3.75, 4.0, 4.5, 5.5, 6.0, 6.5]
)


omega0 = np.array(
    [
        2.77540415e10,
        2.77432909e10,
        2.77326310e10,
        2.77142104e10,
        2.76758227e10,
        2.76196184e10,
        2.75408911e10,
        2.74323625e10,
        2.72930581e10,
        2.71600727e10,
        2.67212682e10,
        2.54085435e10,
        2.44723755e10,
        2.33297370e10,
    ]
)

kappa0 = np.array(
    [
        8.64329301e06,
        9.12750451e06,
        1.04117298e07,
        1.24896070e07,
        1.47630359e07,
        1.80374733e07,
        2.12900056e07,
        2.73854439e07,
        3.03757125e07,
        3.40503258e07,
        4.45854503e07,
        4.37670408e07,
        6.93614881e07,
        1.00632555e08,
    ]
)

kappa_ext = np.array(
    [
        2.15139828e08,
        2.11396591e08,
        2.11539661e08,
        2.16728238e08,
        2.11519383e08,
        2.13091277e08,
        2.12637531e08,
        2.10880411e08,
        2.03932282e08,
        1.99648442e08,
        1.96730230e08,
        1.53050328e08,
        1.53576645e08,
        1.24737482e08,
    ]
)


data = np.concatenate((omega0, kappa0, kappa_ext))
T_arr_fine = np.linspace(T_arr[0], T_arr[-1], 20)

Rex_guess = 50
L0_guess = Rex_guess / kappa_ext[0]
C_guess = 1 / (omega0[0] ** 2 * L0_guess)

# --- fitting with Mattis-Bardeen --- #
pars = Parameters()

pars.add("Tc", value=9.45, min=0, vary=True)
pars.add("a", value=1.85, min=1, vary=True)
pars.add("w00", value=omega0[0], vary=True)


def w0_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    w00 = params["w00"]
    D0 = a * Tc * KB
    w0 = w00 / np.sqrt(fit_T_dep.fL(T, Tc, D0))
    if data is None:
        return w0
    else:
        return data - w0


def w0_T_TTF(params, T, data=None):
    Tc = params["Tc"]
    w00 = params["w00"]
    w0 = w00 / np.sqrt(fit_T_dep.fL1(T, Tc))
    if data is None:
        return w0
    else:
        return data - w0


pars_w0 = pars
fitter = Minimizer(w0_T, pars_w0, fcn_args=(T_arr, omega0))
result = fitter.minimize()

fitter_TTF = Minimizer(w0_T_TTF, pars, fcn_args=(T_arr, omega0))
result_TTF = fitter_TTF.minimize()

fit_pars_w0 = result.params
fit_pars_w0_TTF = result_TTF.params


Tc = fit_pars_w0["Tc"]
a = fit_pars_w0["a"]
w00 = fit_pars_w0["w00"]
w_fit = w0_T(fit_pars_w0, T_arr_fine)
w_TTF = w0_T_TTF(fit_pars_w0_TTF, T_arr_fine)

if verbose:
    print(fit_pars_w0)

# --- plot panel a --- #

ax1.plot(T_arr, omega0 / 2 / np.pi * 1e-9, "o", ms=4, label="Data", color="C0")
ax1.plot(T_arr_fine, w_fit / 2 / np.pi * 1e-9, "-", label="MB", color="C1")
ax1.plot(T_arr_fine, w_TTF / 2 / np.pi * 1e-9, "--", label="TTF", color="C2")
ax1.legend(frameon=False, fontsize=8)
ax1.set_ylabel("$\omega_0 / 2 \pi$ (GHz)")

# --- insert schematic --- #

schem = plt.imread("schematic_circuit_series_lc_direct.png", format="png")
axin = ax1.inset_axes([0.01, 0.1, 0.4, 0.5], transform=ax1.transAxes)
axin.imshow(schem)
axin.axis("off")

# --- fit cavity loss rates --- #


def kex_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    L0 = params["L0"]
    D0 = a * Tc * KB
    L = L0 * fit_T_dep.fL(T, Tc, D0)
    kex = 50 / (L)
    if data is None:
        return kex
    else:
        return kex - data


pars_kex = Parameters()
pars_kex.add("a", value=a, min=1, vary=False)
pars_kex.add("Tc", value=Tc, min=1, vary=False)
pars_kex.add("L0", value=50 / kappa_ext[0], min=0, vary=True)

fitter_kex = Minimizer(kex_T, pars_kex, fcn_args=(T_arr, kappa_ext))
result_kex = fitter_kex.minimize()

fit_pars_kex = result_kex.params
L0 = fit_pars_kex["L0"]

kex_fit = kex_T(result_kex.params, T_arr_fine)

# --- plot panel b --- #

ax2.plot(
    T_arr,
    kappa_ext / 2 / np.pi * 1e-6,
    "^",
    ms=4,
    color="C0",
    label="$\kappa_{{\mathrm{{ext}}}}$",
)
ax2.plot(
    T_arr_fine,
    kex_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{\mathrm{{ext}}}}$ fit",
)

if verbose:
    print(fit_pars_kex)

# --- fit internal loss rates --- #

pars_k0 = Parameters()
pars_k0.add("a", value=a, min=0, vary=False)
pars_k0.add("Tc", value=Tc, min=0, vary=False)
pars_k0.add("L0", value=L0, min=0, vary=False)
pars_k0.add("w00", value=w00, min=0, vary=False)
pars_k0.add("Rd", value=514e3, min=1, vary=False)
pars_k0.add("beta", value=11, min=0, vary=False)
pars_k0.add("Rd0", value=23.56e6, min=0, vary=False)


def k0_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    L0 = params["L0"]
    # RL0=params['RL0']
    Rd = params["Rd"]
    Rd0 = params["Rd0"]
    w00 = params["w00"]
    beta = params["beta"]

    D0 = a * Tc * KB

    w0 = w00 / np.sqrt(fit_T_dep.fL(T, Tc, D0))
    # RL0=D0*L0*np.pi/hbar
    RL0 = D0 * L0 * np.pi / hbar
    # L0=hbar*RL0/D0
    RL = RL0 * fit_T_dep.fR(T, Tc, D0)
    L = L0 * fit_T_dep.fL(T, Tc, D0)
    ZL = 1 / (1 / (1j * w0 * L) + 1 / RL + 1 / (Rd * np.exp(beta / T)) + 1 / Rd0)

    k0 = np.real(ZL) / (np.imag(ZL) / w0)
    if data is None:
        return k0
    else:
        return k0 - data


fitter_k0 = Minimizer(k0_T, pars_k0, fcn_args=(T_arr, kappa0))
result_k0 = fitter_k0.minimize()

fit_pars_k0 = result_k0.params

k0_fit = k0_T(fit_pars_k0, T_arr_fine)

ax2.plot(
    T_arr,
    kappa0 / 2 / np.pi * 1e-6,
    "v",
    ms=4,
    color="C0",
    label="$\kappa_{{0}}$",
)

ax2.plot(
    T_arr_fine,
    k0_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{0}}$ fit",
    label="fit",
)

if verbose:
    print(fit_pars_k0)


# --- plot internal loss rates in the quasi particle limit --- #
fit_pars_k0_QP = result_k0.params
fit_pars_k0_QP["Rd"].value = np.inf
fit_pars_k0_QP["Rd0"].value = np.inf
k0_QP = k0_T(fit_pars_k0_QP, T_arr_fine)

ax2.plot(
    T_arr_fine,
    k0_QP / 2 / np.pi * 1e-6,
    "-",
    color="C1",
)

# --- annotate curves --- #
ax2.annotate(
    "$\kappa_{{\mathrm{{ext}}}}$",
    xy=(2.5, 60),
)

ax2.annotate(
    "$\kappa_{{0}}$",
    xy=(2.5, 1),
)

ax2.annotate(
    "$\kappa_{{0}}^{{\mathrm{{QP}}}}$",
    xy=(3.5, 0.2),
)


# --- customise --- #

ax2.set_yscale("log")
ax2.set_ylim(1e-1, 1.5e2)
ax2.set_ylabel("$\kappa/ 2 \pi$ (MHz)")
ax2.set_xlabel("Temperature (K)")

ax2.legend(frameon=False, fontsize=8, loc="lower right")

# --- annotate sublabels --- #
annotate_params = dict(xy=(-0.2, 0.9), xycoords="axes fraction", annotation_clip=False)
ax1.annotate(r"\textbf{(a)}", **annotate_params)
ax2.annotate(r"\textbf{(b)}", **annotate_params)

# --- save --- #
fig.savefig("figure2_series_lc_direct.pdf", dpi=600)
